@@ -107,28 +107,28 @@ class Drone:
 
         # input: angular position command
         # output: angular velocity
-        self.Kp_ang_pos = np.array([0.5, 0.5, 1.0])
+        self.Kp_ang_pos = np.array([2.0, 2.0, 2.0])
         self.Ki_ang_pos = np.array([0.0, 0.0, 0.0])
         self.Kd_ang_pos = np.array([0.0, 0.0, 0.0])
-        self.lim_ang_pos = np.array([2.0, 2.0, 2.0])
+        self.lim_ang_pos = np.array([3.0, 3.0, 3.0])
 
         # input: linear velocity command
         # output: angular position
-        self.Kp_lin_vel = np.array([2.0, 2.0])
-        self.Ki_lin_vel = np.array([0.0, 0.0])
+        self.Kp_lin_vel = np.array([0.8, 0.8])
+        self.Ki_lin_vel = np.array([0.1, 0.1])
         self.Kd_lin_vel = np.array([0.5, 0.5])
-        self.lim_lin_vel = np.array([0.6, 0.6])
+        self.lim_lin_vel = np.array([0.4, 0.4])
 
         # input: linear position command
         # outputs: linear velocity
         self.Kp_lin_pos = np.array([1.0, 1.0])
         self.Ki_lin_pos = np.array([0.0, 0.0])
         self.Kd_lin_pos = np.array([0.0, 0.0])
-        self.lim_lin_pos = np.array([1.0, 1.0])
+        self.lim_lin_pos = np.array([0.5, 0.5])
 
         # height controllers
-        z_pos_PID = PID(3.0, 0.0, 0.0, 1.0, self.ctrl_period)
-        z_vel_PID = PID(0.2, 1.25, 0.0, 1.0, self.ctrl_period)
+        z_pos_PID = PID(1.0, 0.0, 0.0, 1.0, self.ctrl_period)
+        z_vel_PID = PID(0.2, 1.25, 0.03, 0.3, self.ctrl_period)
         self.z_PIDs = [z_vel_PID, z_pos_PID]
         self.PIDs = []
 
@@ -151,9 +151,6 @@ class Drone:
         self.rpm = np.zeros((4))
         self.pwm = np.zeros((4))
 
-        for PID in self.PIDs:
-            PID.reset()
-
         self.p.resetBasePositionAndOrientation(self.Id, self.start_pos, self.start_orn)
         self.update_state()
 
@@ -200,9 +197,10 @@ class Drone:
         pwm = np.matmul(self.motor_map, cmd)
 
         # deal with motor saturations
-        pwm = np.clip(pwm, 0.0, np.inf)
-        if max := np.max(pwm)> 1.0:
-            pwm = pwm / max
+        if (high := np.max(pwm)) > 1.0:
+            pwm /= high
+        if (low := np.min(pwm)) < 0.05:
+            pwm += (1.0 - pwm) / (1.0 - low) * (0.05 - low)
 
         return pwm
 
@@ -338,6 +336,9 @@ class Drone:
             )
             self.PIDs = [ang_vel_PID, ang_pos_PID, lin_vel_PID, lin_pos_PID]
 
+        for controller in self.PIDs:
+            controller.reset()
+
     def update_control(self):
         """runs through PID controllers"""
         output = None
